"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CaptureInterface = void 0;
var cap_1 = require("cap");
var events_1 = require("events");
var frame_processing_1 = require("./frame-processing");
var models_1 = require("./models");
var pako_1 = __importDefault(require("pako"));
var PROTOCOL = cap_1.decoders.PROTOCOL;
var FILTER = "tcp portrange 54992-54994 or tcp portrange 55006-55007 or tcp portrange 55021-55040 or tcp portrange 55296-55551";
var BYTE = 1;
var KILOBYTE = 1024 * BYTE;
var MEGABYTE = 1024 * KILOBYTE;
var FRAME_HEADER_SIZE = 40;
var SEG_HEADER_SIZE = 16;
var IPC_HEADER_SIZE = 16;
var CaptureInterface = /** @class */ (function (_super) {
    __extends(CaptureInterface, _super);
    function CaptureInterface() {
        var _this = _super.call(this) || this;
        _this._cap = new cap_1.Cap();
        _this._buf = Buffer.alloc(65535);
        return _this;
    }
    CaptureInterface.prototype.open = function (deviceIdentifier) {
        var device = cap_1.Cap.findDevice(deviceIdentifier);
        this._cap.open(device, FILTER, 10 * MEGABYTE, this._buf);
        this._cap.setMinBytes && this._cap.setMinBytes(0);
        this._registerInternalHandlers();
    };
    CaptureInterface.prototype._registerInternalHandlers = function () {
        var _this = this;
        this._cap.on("packet", function (nBytes) {
            // The total buffer is way bigger than the relevant data, so we trim that first.
            var payload = _this._buf.slice(0, nBytes);
            var ret = cap_1.decoders.Ethernet(payload);
            if (ret.info.type === PROTOCOL.ETHERNET.IPV4) {
                ret = cap_1.decoders.IPV4(payload, ret.offset);
                // The info object is destroyed once we decode the TCP data from the packet payload.
                var srcAddr = ret.info.srcaddr;
                var dstAddr = ret.info.dstaddr;
                if (ret.info.protocol === PROTOCOL.IP.TCP) {
                    var datalen = ret.info.totallen - ret.hdrlen;
                    ret = cap_1.decoders.TCP(payload, ret.offset);
                    datalen -= ret.hdrlen;
                    if ((ret.info.flags & 8) === 0)
                        return; // Only TCP PSH has actual data.
                    var childFramePayload = payload.slice(payload.length - datalen);
                    var cfHeaderPayload = childFramePayload.slice(0, FRAME_HEADER_SIZE);
                    var frameHeader = frame_processing_1.parseFrameHeader(cfHeaderPayload);
                    if (!frame_processing_1.isMagical(frameHeader))
                        return;
                    var packet = {
                        source: {
                            address: srcAddr,
                            port: ret.info.srcport,
                        },
                        destination: {
                            address: dstAddr,
                            port: ret.info.dstport,
                        },
                        childFrame: {
                            header: frameHeader,
                            segments: [],
                        },
                    };
                    // Decompress the segments, if necessary.
                    var remainder = childFramePayload.slice(FRAME_HEADER_SIZE);
                    if (frameHeader.isCompressed) {
                        try {
                            var decompressed = pako_1.default.inflate(remainder);
                            remainder = Buffer.from(decompressed.buffer);
                        }
                        catch (err) {
                            // This will happen if the packet contents are encrypted.
                            if (err === "incorrect header check") {
                                return;
                            }
                        }
                    }
                    var offset = 0;
                    for (var i = 0; i < frameHeader.segmentCount; i++) {
                        var segmentPayload = remainder.slice(offset);
                        var segmentHeader = frame_processing_1.parseSegmentHeader(segmentPayload);
                        var ipcHeader = void 0;
                        var ipcData = void 0;
                        if (segmentHeader.segmentType === models_1.SegmentType.Ipc) {
                            var ipcPayload = remainder.slice(offset + SEG_HEADER_SIZE);
                            ipcHeader = frame_processing_1.parseIpcHeader(ipcPayload);
                            ipcData = Buffer.alloc(segmentHeader.size - SEG_HEADER_SIZE - IPC_HEADER_SIZE);
                            ipcPayload.copy(ipcData, 0, IPC_HEADER_SIZE);
                        }
                        var segment = {
                            header: segmentHeader,
                            ipcHeader: ipcHeader,
                            ipcData: ipcData,
                        };
                        packet.childFrame.segments.push();
                        _this.emit("message", "unknown", segment);
                        offset += segmentHeader.size;
                    }
                    _this.emit("packet", packet);
                }
            }
        });
    };
    return CaptureInterface;
}(events_1.EventEmitter));
exports.CaptureInterface = CaptureInterface;
//# sourceMappingURL=pcap-ffxiv.js.map